VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Rigol1052v2"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'///////////////////////////////////////////////////////////////
'// for rigol 1000 series (1052e/1102e)
'// created (16 August 2011)
'//
'// dependency:
'// - visa32.dll
'// - visa32.bas
'// - Rigol1052v2.bas
'//
'// function:
'// - Connect
'// - Connected
'// - Disconnect
'//
'// - SendCommand
'// - GetCommand
'// - WaitCommand
'// - GetData
'//
'// - GetVersion
'// - ClosestTimeScale
'// - Measure
'// - SampleRate
'// - Translate
'//
'// property:
'// - ChanDisp
'// - ChanScale
'// - ChanOffset
'//
'// - TrigStat
'// - TrigMode
'// - TrigSweep
'// - TrigSource
'//
'// - TimeMode
'// - TimeScale
'//
'// - MemLength
'// - MemLength
'//
'// - MathMode
'// - SendWaitCommand
'//
'///////////////////////////////////////////////////////////////

'to buffer rigol return value
Private Const rigBufSize As Long = 2000000
Dim buf As String

'rigol device id
Dim hWnd As Long
Dim defrm As Long

'rigol fw version/variation
Dim myVer As Long

'if true, call WaitCommand on each property Let call
Public SendWaitCommand As Boolean



Private Sub Class_Initialize()
    hWnd = 0
    defrm = 0
    SendWaitCommand = True

    'fill buffer with data
    buf = String(rigBufSize, Chr(1))
End Sub

Private Sub Class_Terminate()
    buf = ""
End Sub



'///////////////////////////////////////////////////////////////
'// basic functions
'///////////////////////////////////////////////////////////////



Public Function Connect() As Boolean
    Dim list As Long
    Dim nmatches As Long
    Dim matches As String * 200
    Dim vi As Long
    Dim res As Long
    Dim tmp As String

    'return if already connected
    If hWnd <> 0 Then
        Connect = True
        Exit Function
    End If

    'acquire USB resource of visa
    res = viOpenDefaultRM(defrm)
    res = viFindRsrc(defrm, "USB?*", list, nmatches, matches)

    'open the equipment
    res = viOpen(defrm, matches, 0, 0, vi)
    hWnd = vi

    'connect failed
    If (hWnd = 0) Then
        Disconnect
        Connect = False

    'connect successfull
    Else

        'rough check if rigol already occupied by other app and assign rigol version
        clearBuffer
        myVer = GetVersion

        'rigol already occupied, abort connection
        If myVer <= 0 Then
            Disconnect
            Exit Function
        End If

        'connect successfull, lock rigol key
        SendCommand ":KEY:LOCK ENAB"
        Connect = True
    End If
End Function

Public Function Connected() As Boolean
    If hWnd Then Connected = True
End Function

Public Sub Disconnect()

    're-enable rigol key
    If Connected Then SendCommand ":KEY:LOCK DIS"

    'close the resource
    viClose hWnd
    viClose defrm

    'reset global
    defrm = 0
    hWnd = 0
End Sub



'///////////////////////////////////////////////////////////////
'// command and read functions
'///////////////////////////////////////////////////////////////



'send visa command to rigol. return -1 if not connected
Public Function SendCommand(cmd As String) As Long
    SendCommand = -1
    If Not Connected Then Exit Function
    SendCommand = viVPrintf(hWnd, cmd + Chr(10), 0)
End Function

'return data for recently sent command
Public Function GetCommand(Optional ByVal lpLength As Long = 0) As String
    Dim i As Long
    If Not Connected Then Exit Function

    'get the return value
    viVScanf hWnd, "%t", buf

    If lpLength > 0 Then
        GetCommand = Left(buf, lpLength)
    Else
        lpLength = InStr(1, buf, Chr(0), vbBinaryCompare)
        If lpLength > 0 Then GetCommand = Left(buf, lpLength - 1)
    End If
End Function

'wait until dso ready for the next command
Public Sub WaitCommand()
    Measure CH1_IDX, RIG_MEASURE_FREQUENCY
End Sub

Public Function GetData(chId As Long, dat() As Byte, lpLength As Long) As Long
    Dim staRead As Long
    Dim i As Long

    'check index and send command
    If chId < CH1_IDX And chId >= MATH_IDX Then Exit Function
    If SendCommand(":WAV:DATA? CHAN" + CStr(chId)) < 0 Then Exit Function

    'channel wave data
    'during running = 600 bytes data returned
    'during stop = 16384 bytes data returned
    'normal buffer single channel (16384 bytes data returned)
    'normal buffer double channel (16384 bytes data returned shared)
    'long buffer single channel (524288 bytes data returned shared)
    'long buffer double channel (1048576 bytes data returned shared)
    'based on sigxcpu advice

    'get the return value
    clearBuffer
    viVScanf hWnd, "%t", buf

    'get data length (beware lpLength is modified!)
    If lpLength <= 0 Then
        lpLength = InStr(1, buf, Chr(0), vbBinaryCompare)
        ReDim dat(lpLength - 1)
    End If

    If lpLength > 0 Then

        'compensate for newer version Rigol FW
        staRead = IIfLong(myVer = 1, 1, 11)

        'convert string data into byte data
        For i = staRead To lpLength + staRead - 1
            dat(i - staRead) = Asc(Mid(buf, i, 1))
        Next i
    End If

    GetData = lpLength
End Function

Public Function GetVersion() As Long
    Dim tmp As String
    If SendCommand(":CHAN1:DISP?") < 0 Then Exit Function
    tmp = GetCommand

    Select Case tmp
    Case "ON": GetVersion = 1
    Case "OFF": GetVersion = 1
    Case "1": GetVersion = 2
    Case "0": GetVersion = 2
    End Select
End Function



'get nearest acceptable time scale value
Public Function ClosestTimeScale(ByVal val As Double) As Double
    Const MIN_SCALE As Double = 0.000000002
    Const MAX_SCALE As Double = 50

    Dim i As Long

    'boundary check
    If val < MIN_SCALE Then val = MIN_SCALE
    If val > MAX_SCALE Then val = MAX_SCALE

    Do Until val < 1
        val = val / 10
        i = i + 1
    Loop

    Do Until val > 1
        val = val * 10
        i = i - 1
    Loop

    If val < 1.5 Then
        val = 1
    ElseIf val < 3.5 Then
        val = 2
    ElseIf val < 7.5 Then
        val = 5
    Else
        val = 1
        i = i + 1
    End If

    val = val * 10 ^ i
    If val > MAX_SCALE Then val = MAX_SCALE
    If val < MIN_SCALE Then val = MIN_SCALE

    ClosestTimeScale = val
End Function

'get some dso measurement
Public Function Measure(chId As Long, measureType As Long) As Double
    Dim tmp As String
    If chId < CH1_IDX Or chId >= MATH_IDX Then Exit Function

    tmp = ":MEAS:"
    Select Case measureType
    Case RIG_MEASURE_FREQUENCY: tmp = tmp + "FREQ?"
    Case RIG_MEASURE_VPP: tmp = tmp + "VPP?"
    Case RIG_MEASURE_VMAX: tmp = tmp + "VMAX?"
    Case RIG_MEASURE_VMIN: tmp = tmp + "VMIN?"
    Case Else: tmp = tmp + "CLEAR"
    End Select

    tmp = tmp + " CHAN" + CStr(chId)
    If SendCommand(tmp) >= 0 Then
        On Error Resume Next
        Measure = CDbl(GetCommand)
    End If
End Function

'if ch1 or ch2, return clock in Hz, if time_idx, return sampling time in sec/point
Public Function SampleRate(chId As Long) As Double

    'channel sample rate (samples/sec)
    If chId >= CH1_IDX And chId < MATH_IDX Then
        If SendCommand(":ACQ:SAMP? CHAN" + CStr(chId)) >= 0 Then SampleRate = CDbl(GetCommand)

    'timescale sample rate between points on screen (s)
    ElseIf chId = TIME_IDX Then
        SampleRate = TimeScale() * 12 / RIG_DISP_NUM
    End If
End Function


'///////////////////////////////////////////////////////////////
'// data byte value conversion:
'// 25 = max value, 225 = min value (tebalik on the screen)
'// Rigol Specific Conversion (from application)
'// 25 = RIG_RAW_MAX, 225 = RIG_RAW_MIN, 125 = 0 (center)
'//
'// the absolute data value limit
'// out of boundary (top) = 15
'// out of boundary (bottom) = 240
'//
'///////////////////////////////////////////////////////////////

Public Function Translate(val As Double, min As Double, max As Double, convType As Long) As Double
    Dim res As Double

    'convert rigol to volt value (val is any rigol byte value) recommended min max = 4 * ChanScale
    If convType = RIG_RAW_TO_REAL Then
        res = CDbl((max - min) * (CLng(val) - RIG_RAW_MAX)) / (RIG_RAW_MIN - RIG_RAW_MAX)
        res = CDbl(max) - res

    'convert volt to rigol value (val, min, max can be any valid volt value)
    ElseIf convType = RIG_REAL_TO_RAW Then
        res = (((max - val) * (RIG_RAW_MIN - RIG_RAW_MAX)) / (max - min)) + RIG_RAW_MAX
        res = Int(res) And 255
    End If

    Translate = res
End Function

'set buffer to zero length string retaining buffer size
Private Sub clearBuffer()
    Mid(buf, 1, 1) = Chr(0)
End Sub

Private Function formatNumber(val As Double) As String
    formatNumber = Format(val, "0.############0")
End Function


'///////////////////////////////////////////////////////////////
'// properties
'///////////////////////////////////////////////////////////////



'///////////////////////////////////////////////////////////////
'// channel display, return/set true or false
'// accept CH1, CH2, MATH IDX

Public Property Get ChanDisp(chId As Long) As Boolean
    Dim tmp As String

    'get channel on/off
    If chId >= CH1_IDX And chId < MATH_IDX Then
        tmp = ":CHAN" + CStr(chId) + ":DISP?"

    'get math on/off
    ElseIf chId = MATH_IDX Then
        tmp = ":MATH:DISP?"
    End If

    'get display true or false
    If SendCommand(tmp) >= 0 Then
        tmp = GetCommand
        If tmp = "ON" Then ChanDisp = True
        If tmp = "1" Then ChanDisp = True
    End If
End Property

Public Property Let ChanDisp(chId As Long, ByVal vNewValue As Boolean)
    Dim tmp As String

    'set channel on/off
    If chId >= CH1_IDX And chId < MATH_IDX Then
        tmp = ":CHAN" + CStr(chId) + ":DISP "
        If vNewValue Then tmp = tmp + "ON"
        If Not vNewValue Then tmp = tmp + "OFF"

    'set math on/off
    ElseIf chId = MATH_IDX Then
        tmp = ":MATH:DISP "
        If vNewValue Then tmp = tmp + "ON"
        If Not vNewValue Then tmp = tmp + "OFF"
    End If

    'throw the command
    If tmp <> "" Then
        If SendCommand(tmp) >= 0 Then
            If SendWaitCommand Then WaitCommand
        End If
    End If
End Property


'///////////////////////////////////////////////////////////////
'// channel time scale, return V/div
'// accept CH1, CH2 IDX

Public Property Get ChanScale(chId As Long) As Double
    If SendCommand(":CHAN" + CStr(chId) + ":SCAL?") >= 0 Then ChanScale = CDbl(GetCommand)
End Property

Public Property Let ChanScale(chId As Long, ByVal vNewValue As Double)
    If SendCommand(":CHAN" + CStr(chId) + ":SCAL " + formatNumber(vNewValue)) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property



'///////////////////////////////////////////////////////////////
'// channel offset value
'// accept CH1, CH2, MATH, TIME IDX

Public Property Get ChanOffset(chId As Long) As Double
    Dim tmp As String

    'get channel offset
    If chId >= CH1_IDX And chId < MATH_IDX Then
        tmp = ":CHAN" + CStr(chId) + ":OFFS?"

    'get time offset
    ElseIf chId = TIME_IDX Then
        tmp = TimeMode
        If tmp = "MAIN" Then tmp = ":TIM:OFFS?"
        If tmp = "DELAYED" Then tmp = ":TIM:DEL:OFFS?"

    'get trig offset with unsupported escape
    ElseIf chId = TRIG_IDX Then
        tmp = TrigMode
        If tmp = "SLOPE" Then Exit Property
        If tmp = "PATTERN" Then Exit Property
        If tmp = "DURATION" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property
        tmp = ":TRIG:" + tmp + ":LEV?"
    End If

    'get offset value
    If tmp <> "" Then
        If SendCommand(tmp) >= 0 Then ChanOffset = CDbl(GetCommand)
    End If
End Property

Public Property Let ChanOffset(chId As Long, ByVal vNewValue As Double)
    Dim tmp As String

    'set channel offset
    If chId >= CH1_IDX And chId < MATH_IDX Then
        tmp = ":CHAN" + CStr(chId) + ":OFFS " + formatNumber(vNewValue)

    'set time offset
    ElseIf chId = TIME_IDX Then
        tmp = TimeMode
        If tmp = "MAIN" Then tmp = ":TIM:OFFS "
        If tmp = "DELAYED" Then tmp = ":TIM:DEL:OFFS "
        tmp = tmp + formatNumber(vNewValue)

    'set trig offset with unsupported mode
    ElseIf chId = TRIG_IDX Then
        tmp = TrigMode
        If tmp = "SLOPE" Then Exit Property
        If tmp = "PATTERN" Then Exit Property
        If tmp = "DURATION" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property
        tmp = ":TRIG:" + tmp + ":LEV " + formatNumber(vNewValue)
    End If

    If tmp <> "" Then
        If SendCommand(tmp) >= 0 Then
            If SendWaitCommand Then WaitCommand
        End If
    End If
End Property



'///////////////////////////////////////////////////////////////
'// trigger status
'// return RUN, STOP, T'D, WAIT, AUTO
'// accept RUN, STOP, AUTO

Public Property Get TrigStat() As String
    If SendCommand(":TRIGGER:STAT?") >= 0 Then TrigStat = GetCommand
End Property

Public Property Let TrigStat(ByVal vNewValue As String)
    If vNewValue <> "STOP" Then vNewValue = "RUN"
    If SendCommand(":" + vNewValue) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property



'///////////////////////////////////////////////////////////////
'// trigger mode
'// return/accept EDGE, PULSE, VIDEO, SLOPE, PATTERN, DURATION, ALTERNATION

Public Property Get TrigMode() As String
    If SendCommand(":TRIG:MODE?") >= 0 Then TrigMode = GetCommand
End Property

Public Property Let TrigMode(ByVal vNewValue As String)
    If SendCommand(":TRIG:MODE " + vNewValue) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property



'///////////////////////////////////////////////////////////////
'// trigger sweep mode
'// return/accept AUTO, NORMAL, SINGLE

Public Property Get TrigSweep() As String
    Dim tmp As String
    tmp = TrigMode

    'video and alternation is unsupported mode
    If tmp <> "" Then
        If tmp = "VIDEO" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property
        If SendCommand(":TRIG:" + tmp + ":SWE?") >= 0 Then TrigSweep = GetCommand
    End If
End Property

Public Property Let TrigSweep(ByVal vNewValue As String)
    Dim tmp As String
    tmp = TrigMode

    'video and alternation is unsupported mode
    If tmp <> "" Then
        If tmp = "VIDEO" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property
        If SendCommand(":TRIG:" + tmp + ":SWE " + vNewValue) >= 0 Then
            If SendWaitCommand Then WaitCommand
        End If
    End If
End Property



'///////////////////////////////////////////////////////////////
'// trigger source
'// return/accept CHAN1, CHAN2, CHANx, EXT, ACL, DIG D0~D15

Public Property Get TrigSource() As String
    If SendCommand(":TRIG:" + TrigMode + ":SOUR?") >= 0 Then TrigSource = GetCommand
End Property

Public Property Let TrigSource(ByVal vNewValue As String)
    If SendCommand(":TRIG:" + TrigMode + ":SOUR " + vNewValue) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property



'///////////////////////////////////////////////////////////////
'// time mode
'// return/accept MAIN, DELAYED

Public Property Get TimeMode() As String
    If SendCommand(":TIM:MODE?") >= 0 Then TimeMode = GetCommand
End Property

Public Property Let TimeMode(ByVal vNewValue As String)
    If SendCommand(":TIM:MODE " + vNewValue) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property



'///////////////////////////////////////////////////////////////
'// channel time scale value in sec/div

Public Property Get TimeScale() As Double
    Dim tmp As String

    tmp = TimeMode
    If tmp = "MAIN" Then tmp = ":TIM:SCAL?"
    If tmp = "DELAYED" Then tmp = ":TIM:DEL:SCAL?"

    If tmp <> "" Then
        If SendCommand(tmp) >= 0 Then TimeScale = CDbl(GetCommand)
    End If
End Property

Public Property Let TimeScale(ByVal vNewValue As Double)
    Dim tmp As String

    tmp = TimeMode
    If tmp = "MAIN" Then tmp = ":TIM:SCAL " + formatNumber(ClosestTimeScale(vNewValue))
    If tmp = "DELAYED" Then tmp = ":TIM:DEL:SCAL " + formatNumber(ClosestTimeScale(vNewValue))

    If tmp <> "" Then
        If SendCommand(tmp) >= 0 Then
            If SendWaitCommand Then WaitCommand
        End If
    End If
End Property



'///////////////////////////////////////////////////////////////
'// memory depth
'// return/accept LONG or NORMAL

Public Property Get MemDepth() As String
    If SendCommand(":ACQ:MEMD?") >= 0 Then MemDepth = GetCommand
End Property

Public Property Let MemDepth(ByVal vNewValue As String)
    If SendCommand(":ACQ:MEMD " + vNewValue) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property



'///////////////////////////////////////////////////////////////
'// memory depth
'// return/accept MAXIMUM or NORMAL

Public Property Get MemLength() As String
    If SendCommand(":WAVEFORM:POINTS:MODE?") >= 0 Then MemLength = GetCommand
End Property

Public Property Let MemLength(ByVal vNewValue As String)
    If SendCommand(":WAVEFORM:POINTS:MODE " + vNewValue) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property



'///////////////////////////////////////////////////////////////
'// math mode
'// return/accept A+B, A-B, A*B/AB, FFT

Public Property Get MathMode() As String
    If SendCommand(":MATH:OPER?") >= 0 Then MathMode = GetCommand
End Property

Public Property Let MathMode(ByVal vNewValue As String)
    If SendCommand(":MATH:OPER " + vNewValue) >= 0 Then
        If SendWaitCommand Then WaitCommand
    End If
End Property
