VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Rigol"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'///////////////////////////////////////////////////////////////
'// for rigol 1052/1102e (support up to 8 channel)
'// created (October 12, 2010)
'//
'// dependency:
'// - rigol.bas
'// - visa32.bas
'// - visa32.dll
'//
'// function:
'// - Connect
'// - Connected
'// - Disconnect
'// - Delay
'// - Command
'// - Read
'// - Flush
'//
'// - Increment
'// - Decrement
'// - Data
'// - RigolValue
'// - RealValue
'// - ClosestTimeScale
'//
'// property:
'// - Sampling
'// - State
'// - Mode
'// - Channel
'// - Offset
'// - Skala
'// - Sweep
'// - Time
'// - Source
'// - Math
'//
'//   (read/write only)
'// - Measure
'// - DataLength
'//
'///////////////////////////////////////////////////////////////



Option Explicit

Private Const rigBufSize As Long = 2000000
Private buf As String

Private hWnd As Long
Private defrm As Long

Private trigView As Boolean

Private Sub Class_Initialize()
    hWnd = 0
    defrm = 0
    trigView = True
    Flush
End Sub

Private Sub Class_Terminate()
    buf = ""
End Sub



'///////////////////////////////////////////////////////////////
'// basic properties
'///////////////////////////////////////////////////////////////

Public Function Connect() As Boolean
    Dim list As Long
    Dim nmatches As Long

    Dim res As Long
    Dim tmp As String

    'equipment ID.
    Dim matches As String * 200
    Dim vi As Long

    'return if already connected
    If hWnd <> 0 Then
        Connect = True
        Exit Function
    End If

    'acquire USB resource of visa
    res = viOpenDefaultRM(defrm)
    res = viFindRsrc(defrm, "USB?*", list, nmatches, matches)

    'open the equipment
    res = viOpen(defrm, matches, 0, 0, vi)
    hWnd = vi

    'connect failed
    If (hWnd = 0) Then
        Disconnect
        Connect = False

    'connect successfull
    Else

        'rough check if rigol already occupied by other app
        Flush
        Command ":CHAN1:DISP?"
        tmp = Read

        'rigol already occupied, abort connection
        If tmp = "" Then
            Disconnect
            Exit Function
        End If

        'connect successfull, lock rigol key
        Command ":KEY:LOCK ENAB"
        Connect = True
    End If
End Function

Public Function Connected() As Boolean
    If hWnd Then Connected = True
End Function

Public Sub Disconnect()

    're-enable rigol key
    If Connected Then
        Command ":KEY:LOCK DIS"
    End If

    'close the resource
    viClose hWnd
    viClose defrm

    'reset global
    defrm = 0
    hWnd = 0
End Sub

'// delay to enable rigol to update its info
Public Sub Delay(Optional ms As Long = 20)
    Dim t As Double
    Dim c As Double
    t = Timer

    c = ms / 1000#
    Do Until Timer - t > c
        If Timer < t Then t = Timer
    Loop
End Sub

Public Function Command(cmd As String) As Long
    If Not Connected Then
        Command = -1
        Exit Function
    End If

    'send the command
    Command = viVPrintf(hWnd, cmd + Chr(10), 0)
End Function

Public Function Read(Optional ByVal retLen As Long = 0) As String
    Dim i As Long
    If Not Connected Then Exit Function

    'get the return value
    viVScanf hWnd, "%t", buf

    If retLen > 0 Then
        Read = Left(buf, retLen)
    Else
        retLen = InStr(1, buf, Chr(0), vbBinaryCompare)
        If retLen > 0 Then Read = Left(buf, retLen - 1)
    End If
End Function

'reset buffer
Public Sub Flush()
    buf = String(rigBufSize, Chr(1))
End Sub



'///////////////////////////////////////////////////////////////
'// rigol properties
'///////////////////////////////////////////////////////////////



'///////////////////////////////////////////////////////////////
'// return sampling rate

Public Property Get Sampling(idx As Long) As Double

    'channel sample rate (samples/sec)
    If idx >= CHA1_IDX And idx < MATH_IDX Then
        If Command(":ACQ:SAMP? CHAN" + CStr(idx)) >= 0 Then
            Sampling = CDbl(Read)
        End If
        
    'timescale sample rate between points on screen (s)
    ElseIf idx = TIME_IDX Then
        Sampling = Skala(TIME_IDX) * 12 / NORM_RIG_MEM
    End If
End Property



'///////////////////////////////////////////////////////////////
'// trigger status
'// return RUN, STOP, T'D, WAIT, AUTO
'// accept RUN, STOP, AUTO

Public Property Get state() As String
    If Command(":TRIGGER:STAT?") >= 0 Then
        state = Read
    End If
End Property

Public Property Let state(ByVal vNewValue As String)
    Command ":" + vNewValue
End Property



'///////////////////////////////////////////////////////////////
'// trigger mode
'// return/accept EDGE, PULSE, VIDEO, SLOPE, PATTERN, DURATION, ALTERNATION

Public Property Get mode() As String
    If Command(":TRIG:MODE?") >= 0 Then
        mode = Read
    End If
End Property

Public Property Let mode(ByVal vNewValue As String)
    Command ":TRIG:MODE " + vNewValue
End Property



'///////////////////////////////////////////////////////////////
'// channel display on or off

Public Property Get Channel(idx As Long) As Boolean
    Dim tmp As String

    If idx <> TRIG_IDX Then
        If Not Connected Then Exit Property

        'get channel on/off
        If idx >= CHA1_IDX And idx < MATH_IDX Then
            Command ":CHAN" + CStr(idx) + ":DISP?"
            tmp = Read

            If tmp = "ON" Then Channel = True
            If tmp = "1" Then Channel = True

        'get math on/off
        ElseIf idx = MATH_IDX Then
            Command ":MATH:DISP?"
            tmp = Read

            If tmp = "ON" Then Channel = True
            If tmp = "1" Then Channel = True
        End If

    'get (simulated) trigger view on or off
    Else
        Channel = trigView
    End If
End Property

Public Property Let Channel(idx As Long, ByVal vNewValue As Boolean)
    Dim tmp As String

    If idx <> TRIG_IDX Then
        If Not Connected Then Exit Property

        'set channel on/off
        If idx >= CHA1_IDX And idx < MATH_IDX Then
            tmp = ":CHAN" + CStr(idx) + ":DISP "
            If vNewValue Then tmp = tmp + "ON"
            If Not vNewValue Then tmp = tmp + "OFF"

        'set math on/off
        ElseIf idx = MATH_IDX Then
            tmp = ":MATH:DISP "
            If vNewValue Then tmp = tmp + "ON"
            If Not vNewValue Then tmp = tmp + "OFF"
        End If

        'throw the command
        Command tmp

    'set trigger view on/off
    Else
        trigView = vNewValue
    End If
End Property



'///////////////////////////////////////////////////////////////
'// channel offset value

Public Property Get offset(idx As Long) As Double
    Dim tmp As String
    If Not Connected Then Exit Property

    'get channel offset
    If idx >= CHA1_IDX And idx < MATH_IDX Then
        Command ":CHAN" + CStr(idx) + ":OFFS?"

    'get time offset
    ElseIf idx = TIME_IDX Then
        tmp = Time
        If tmp = "MAIN" Then tmp = ":TIM:OFFS?"
        If tmp = "DELAYED" Then tmp = ":TIM:DEL:OFFS?"
        Command tmp

    'get trig offset
    ElseIf idx = TRIG_IDX Then
        tmp = mode

        'unsupported mode
        If tmp = "SLOPE" Then Exit Property
        If tmp = "PATTERN" Then Exit Property
        If tmp = "DURATION" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property

        Command ":TRIG:" + tmp + ":LEV?"

    'unsupported channel return zero
    Else
        Exit Property
    End If

    offset = CDbl(Read)
End Property

Public Property Let offset(idx As Long, ByVal vNewValue As Double)
    Dim tmp As String
    If Not Connected Then Exit Property

    'set channel offset
    If idx >= CHA1_IDX And idx < MATH_IDX Then
        Command ":CHAN" + CStr(idx) + ":OFFS " + CStr(vNewValue)

    'set time offset
    ElseIf idx = TIME_IDX Then
        tmp = Time
        If tmp = "MAIN" Then tmp = ":TIM:OFFS "
        If tmp = "DELAYED" Then tmp = ":TIM:DEL:OFFS "
        Command tmp + CStr(vNewValue)

    'set trig offset
    ElseIf idx = TRIG_IDX Then
        tmp = mode

        'unsupported mode
        If tmp = "SLOPE" Then Exit Property
        If tmp = "PATTERN" Then Exit Property
        If tmp = "DURATION" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property

        Command ":TRIG:" + tmp + ":LEV " + CStr(vNewValue)

    'unsupported channel return zero
    Else
        Exit Property
    End If
End Property



'///////////////////////////////////////////////////////////////
'// channel/time scale value

Public Property Get Skala(idx As Long) As Double
    Dim tmp As String
    If Not Connected Then Exit Property

    'get channel scale
    If idx >= CHA1_IDX And idx < MATH_IDX Then
        Command ":CHAN" + CStr(idx) + ":SCAL?"

    'get time scale
    ElseIf idx = TIME_IDX Then
        tmp = Time
        If tmp = "MAIN" Then tmp = ":TIM:SCAL?"
        If tmp = "DELAYED" Then tmp = ":TIM:DEL:SCAL?"
        Command tmp

    'unsupported channel return zero
    Else
        Exit Property
    End If

    Skala = CDbl(Read)
End Property

Public Property Let Skala(idx As Long, ByVal vNewValue As Double)
    Dim tmp As String
    If Not Connected Then Exit Property

    'set channel scale, unit V/div
    If idx >= CHA1_IDX And idx < MATH_IDX Then
        Command ":CHAN" + CStr(idx) + ":SCAL " + CStr(vNewValue)

    'set time scale
    ElseIf idx = TIME_IDX Then
        tmp = Time
        If tmp = "MAIN" Then
            tmp = ":TIM:SCAL " + CStr(ClosestTimeScale(vNewValue))
        ElseIf tmp = "DELAYED" Then
            tmp = ":TIM:DEL:SCAL " + CStr(ClosestTimeScale(vNewValue))
        End If
        Command tmp

    'unsupported channel return zero
    Else
        Exit Property
    End If
End Property

Public Function ClosestTimeScale(ByVal val As Double) As Double
    Const MIN_SCALE As Double = 0.000000002
    Const MAX_SCALE As Double = 50

    Dim i As Long
    
    'boundary check
    If val < MIN_SCALE Then val = MIN_SCALE
    If val > MAX_SCALE Then val = MAX_SCALE
    
    Do Until val < 1
        val = val / 10
        i = i + 1
    Loop

    Do Until val > 1
        val = val * 10
        i = i - 1
    Loop

    If val < 1.5 Then
        val = 1
    ElseIf val < 3.5 Then
        val = 2
    ElseIf val < 7.5 Then
        val = 5
    Else
        val = 1
        i = i + 1
    End If

    val = val * 10 ^ i
    If val > MAX_SCALE Then
        val = MAX_SCALE
    ElseIf val < MIN_SCALE Then
        val = MIN_SCALE
    End If

    ClosestTimeScale = val
End Function



'///////////////////////////////////////////////////////////////
'// trigger sweep mode
'// return/accept AUTO, NORMAL, SINGLE

Public Property Get Sweep() As String
    Dim tmp As String

    tmp = mode
    If tmp <> "" Then

        'unsupported mode
        If tmp = "VIDEO" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property

        Command ":TRIG:" + tmp + ":SWE?"
        Sweep = Read
    End If
End Property

Public Property Let Sweep(ByVal vNewValue As String)
    Dim tmp As String

    tmp = mode
    If tmp <> "" Then

        'unsupported mode
        If tmp = "VIDEO" Then Exit Property
        If tmp = "ALTERNATION" Then Exit Property

        Command ":TRIG:" + tmp + ":SWE " + vNewValue
    End If
End Property



'///////////////////////////////////////////////////////////////
'// time mode
'// return/accept MAIN, DELAYED

Public Property Get Time() As String
    If Command(":TIM:MODE?") >= 0 Then
        Time = Read
    End If
End Property

Public Property Let Time(ByVal vNewValue As String)
    Command ":TIM:MODE " + vNewValue
End Property



'///////////////////////////////////////////////////////////////
'// reporting

'get some dso measurement. jenis:
'1 = frequency, 2 = vpp
'3 = max, 4 = min
'0 & else = clear

Public Property Get Measure(idx As Long, jenis As Long) As Double
    Dim s As String

    'get channel scale
    If idx < CHA1_IDX Or idx >= MATH_IDX Then Exit Property
    
    s = ":MEAS:"
    Select Case jenis

    Case 1: s = s + "FREQ?"
    Case 2: s = s + "VPP?"
    Case 3: s = s + "VMAX?"
    Case 4: s = s + "VMIN?"

    Case Else: s = s + "CLEAR"
    End Select
    s = s + " CHAN" + CStr(idx)

    On Error Resume Next
    If Command(s) >= 0 Then Measure = CDbl(Read)
End Property

'data depth to be returned to pc (refer to data fgunction member for description) mode = "NORMAL", "MAXIMUM"
Public Property Let DataLength(mode As String)
    Command ":WAVEFORM:POINTS:MODE " + mode
End Property



'///////////////////////////////////////////////////////////////
'// trigger source
'// return/accept CHAN1, CHAN2, CHANx, EXT, ACL, DIG D0~D15

Public Property Get Source() As String
    Dim tmp As String
    If Not Connected Then Exit Property

    tmp = mode
    Command ":TRIG:" + tmp + ":SOUR?"

    Source = Read
End Property

Public Property Let Source(ByVal vNewValue As String)
    Dim tmp As String
    If Not Connected Then Exit Property

    tmp = mode
    Command ":TRIG:" + tmp + ":SOUR " + vNewValue
End Property



'///////////////////////////////////////////////////////////////
'// math mode
'// return/accept A+B, A-B, A*B/AB, FFT

Public Property Get Math() As String
    If Command(":MATH:OPER?") >= 0 Then
        Math = Read
    End If
End Property

Public Property Let Math(ByVal vNewValue As String)
    Command ":MATH:OPER " + vNewValue
End Property



'///////////////////////////////////////////////////////////////
'// offMode 0 = (inc/dec offset), 1 = (inc/dec V or s/div)

Public Sub Increment(idx As Long, Optional offMode As Long = 0)
    Select Case offMode
    Case 0: incDecOff idx, "INC"
    Case 1: incDecDiv idx, "INC"
    End Select
End Sub

Public Sub Decrement(idx As Long, Optional offMode As Long = 0)
    Select Case offMode
    Case 0: incDecOff idx, "DEC"
    Case 1: incDecDiv idx, "DEC"
    End Select
End Sub

Private Sub incDecOff(idx As Long, st As String)
    Dim tmp As String

    If idx <= MATH_IDX Then
        tmp = ":KEY:V_POS_" + st

    ElseIf idx = TIME_IDX Then
        tmp = ":KEY:H_POS_" + st

    ElseIf idx = TRIG_IDX Then
        tmp = ":KEY:TRIG_LVL_" + st
        
    Else
        Exit Sub
    End If

    Command tmp
End Sub

Private Sub incDecDiv(idx As Long, st As String)
    Dim tmp As String

    If idx <= MATH_IDX Then
        tmp = ":KEY:V_SCALE_" + st
        
    ElseIf idx = TIME_IDX Then
        tmp = ":KEY:H_SCALE_" + st
        
    Else
        Exit Sub
    End If

    Command tmp
End Sub



'///////////////////////////////////////////////////////////////
'// read data from each channel (return length of data)
'// data must be allocated if lenDat > 0

Public Function data(idx As Long, dat() As Byte, lenDat As Long) As Long
    Dim tml As Long
    Dim i As Long

    If Not Connected Then Exit Function

    'channel wave data
    'during running = 600 bytes data returned
    'during stop = 16384 bytes data returned
    'normal buffer single channel (16384 bytes data returned)
    'normal buffer double channel (16384 bytes data returned shared)
    'long buffer single channel (524288 bytes data returned shared)
    'long buffer double channel (1048576 bytes data returned shared)
    'based on sigxcpu advice

    If idx >= CHA1_IDX And idx < MATH_IDX Then

        'send command
        Command ":WAV:DATA? CHAN" + CStr(idx)

        'get the return value
        viVScanf hWnd, "%t", buf

        'get data length
        tml = lenDat
        If lenDat <= 0 Then tml = InStr(1, buf, Chr(0), vbBinaryCompare)

        If tml > 0 Then
            If lenDat < tml Then ReDim Preserve dat(tml - 1)
            For i = 1 To tml
                dat(i - 1) = asc(mid(buf, i, 1))
            Next i
            data = tml
        End If
    End If
End Function



'///////////////////////////////////////////////////////////////
'// data byte value conversion
'// 25 = max value, 225 = min value (tebalik on the screen)
'// Rigol Specific Conversion (from application)
'// 25 = DSO_MAX_Y, 225 = DSO_MIN_Y, 125 = 0 (center)
'//
'// the absolute data value limit
'// out of boundary (top) = 15
'// out of boundary (bottom) = 240
'//
'///////////////////////////////////////////////////////////////

'// convert real value to rigol raw data scaling
Public Function RigolValue(val As Double, min As Double, max As Double) As Byte
    Dim res As Double

    res = (((max - val) * 200#) / (max - min)) + 25#
    If res > 255 Then
        RigolValue = 255
    ElseIf res >= 0 Then
        RigolValue = CByte(res)
    End If
End Function

'// convert rigol raw data scaling to real (scaled) value
Public Function RealValue(rigValue As Byte, min As Double, max As Double) As Double
    Dim res As Double

    res = CDbl((max - min) * (CLng(rigValue) - 25)) / 200#
    res = CDbl(max) - res

    RealValue = res
End Function
